function [q,r]=deconv(p1,p2)
%% usage: [q,r]=deconv(p1,p2)
%% 
%% If p1, p2 are polynomial coefficients,
%% calculates the quotient q and remainder r
%% of the polynomial division.
%% <pre>Example:
%% >> p1=[1 0 2];p2=[1 -1];
%% >> deconv(p1,p2)
%% r = 3
%% q = [ 1  1 ]</pre>
	len1 = length(p1);
	len2 = length(p2);
	pc = rat(p1);
	if len1<len2
		q = 0;
		r = double(pc);
		return;
	end
	q = zeros(1,len1-len2+1);
	for i=1:length(q)
		q(i) = pc(i)/p2(1);
		pc(i:i+len2-1) -= q(i)*p2;
	end
	while pc(1)==0
		if length(pc)>1
			pc = pc(2:length(pc));
		else
			break;
		end
	end
	r = double(pc);
end